﻿using System;
using System.Reflection;
using System.Runtime.Remoting;

namespace CV4
{
    class Program
    {
        static void Main(string[] args)
        {
            string assemblyPath= "../../../CV4Library/bin/Debug/CV4Library.dll";
            Assembly assembly = Assembly.ReflectionOnlyLoadFrom(assemblyPath);
            Type calc = assembly.GetType("CV4Library.SimpleCalc");

            if (calc != null)
            {
                foreach (MethodInfo m in calc.GetMethods(
                    BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.DeclaredOnly))
                {
                    Console.Write(m.IsPublic ? "public" : "private");
                    Console.Write(" ");
                    Console.Write(m.ReturnType.Name);
                    Console.Write(" ");
                    Console.Write(m.Name);
                    Console.Write("(");
                    bool first = true;

                    foreach (ParameterInfo p in m.GetParameters())
                    {
                        if (!first)
                            Console.Write(", ");
                        else
                            first = false;

                        Console.Write(p.ParameterType.Name);
                        Console.Write(" ");
                        Console.Write(p.Name);
                    }

                    Console.Write(")");
                    Console.WriteLine();
                }
            }

            Console.WriteLine();

            AppDomain.CurrentDomain.AssemblyResolve += (s, e) => Assembly.LoadFrom(assemblyPath);
            AppDomain domain = AppDomain.CreateDomain("MyNewDomain");
            ObjectHandle handle = domain.CreateInstanceFrom(assemblyPath, "CV4Library.SimpleCalc");
            object obj = handle.Unwrap();

            if (RemotingServices.IsTransparentProxy(obj))
            {
                Type type = obj.GetType();
                MethodInfo setXY = type.GetMethod("SetXY");
                MethodInfo showResult = type.GetMethod("ShowResult");
                MethodInfo add = type.GetMethod("Add", BindingFlags.Instance | BindingFlags.NonPublic);

                setXY.Invoke(obj, new object[] { 3, 5 });
                showResult.Invoke(obj, new object[] { "Multiplied result is {0}" });

                int result = (int)add.Invoke(obj, null);
                Console.WriteLine("Square of result of private Add method is: {0}", result * result);
            }

            AppDomain.Unload(domain);
        }
    }
}
